//+------------------------------------------------------------------+
//|                                        Christopher-mq5-robot.mq5 |
//|                                      Copyright 2015, Danielkniaz |
//|           https://www.upwork.com/freelancers/~01ec902808c13f166b |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Danielkniaz"
#property link      "https://www.upwork.com/freelancers/~01ec902808c13f166b"
#property version   "1.01"
//---
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
//---
input int MagicNumber = 77777775;
input double Lot = 0.01;
input int Stoploss = 0;
input int Breakeven= 0;
input int Takeprofit=1000;
input int TrailingStop = 0;
//---
input int ind_qual = 5;
input int ind_len  = 5;
input int ind_countbars=1000;
//---
bool Email = true;
string ind = "Leledc-ExhaustionBar1_mq5";
int ExtTimeOut=10; // time out in seconds between trade operations
//+------------------------------------------------------------------+
//| Expert class                                                     |
//+------------------------------------------------------------------+
class CExpert
  {
protected:
   double            m_adjusted_point;             // point value adjusted for 3 or 5 points
   CTrade            m_trade;                      // trading object
   CSymbolInfo       m_symbol;                     // symbol info object
   CPositionInfo     m_position;                   // trade position object
   CAccountInfo      m_account;                    // account info wrapper
   //--- indicators
   int               m_handle_lelExt;              // Leledc-Exh indicator handle 0
   //--- indicator buffers
   double            m_buff_lelExt0[];             // MACD indicator main buffer
   double            m_buff_lelExt1[];             // MACD indicator signal buffer
   //--- indicator data for processing
   double            m_lelExt0;
   double            m_lelExt1;
   //---
   double            m_traling_stop;
   double            m_take_profit;
public:
                     CExpert(void);
                    ~CExpert(void);
   bool              Init(void);
   void              Deinit(void);
   bool              Processing(void);
   bool              ProcessingTrail(void);
protected:
   bool              InitCheckParameters(const int digits_adjust);
   bool              InitIndicators(void);
   bool              LongClosed(void);
   bool              ShortClosed(void);
   bool              LongModified(void);
   bool              ShortModified(void);
   bool              LongOpened(void);
   bool              ShortOpened(void);
   double            LotSize(void);
  };
//--- global expert
CExpert ResultExpert;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CExpert::CExpert(void) : m_adjusted_point(0),
                         m_handle_lelExt(INVALID_HANDLE),
                         m_lelExt0(0),
                         m_lelExt1(0),
                         m_traling_stop(0),
                         m_take_profit(0)
  {
   ArraySetAsSeries(m_buff_lelExt0,true);
   ArraySetAsSeries(m_buff_lelExt1,true);
  }
CExpert::~CExpert(void){}
//---
bool CExpert::Init(void)
  {
//--- initialize common information
   m_symbol.Name(_Symbol);                   // symbol
   m_trade.SetExpertMagicNumber(MagicNumber);// magic
//--- tuning for 3 or 5 digits
   int digits_adjust=1;
   if(m_symbol.Digits()==3 || m_symbol.Digits()==5)
      digits_adjust=10;
   m_adjusted_point=m_symbol.Point()*digits_adjust;
//--- set default deviation for trading in adjusted points
   m_traling_stop    =TrailingStop*m_adjusted_point;
   m_take_profit     =Takeprofit*m_adjusted_point;
//--- set default deviation for trading in adjusted points
   m_trade.SetDeviationInPoints(2*digits_adjust);
//---
   if(!InitCheckParameters(digits_adjust))
      return(false);
   if(!InitIndicators())
      return(false);
//--- succeed
   return(true);  
  }
CExpert::Deinit(void){  }
bool CExpert::InitCheckParameters(const int digits_adjust)
  {
//--- initial data checks
   if(Takeprofit*digits_adjust<m_symbol.StopsLevel())
     {
      printf("Take Profit must be greater than StopLevel %d",m_symbol.StopsLevel());
      return(false);
     }
   if(TrailingStop*digits_adjust<m_symbol.StopsLevel())
     {
      printf("Trailing Stop must be greater than StopLevel %d",m_symbol.StopsLevel());
      return(false);
     }
//--- warning
   if(Takeprofit<=TrailingStop)
      printf("Warning: Trailing Stop must be less than Take Profit");
//--- succeed
   return(true);
  }
bool CExpert::InitIndicators(void)
  {
//--- create LelExh indicator
   if(m_handle_lelExt==INVALID_HANDLE)
      if((m_handle_lelExt=iCustom(_Symbol,0,ind,  ind_qual,ind_len,ind_countbars))==INVALID_HANDLE){
         printf("Error creating Leledc_ExhaustionBar3_upd_mq5 - 0 indicator");
         return false;
      }
//--- succeed
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------
bool CExpert::ProcessingTrail(void)
  {
   if(m_position.Select(Symbol()))
     {
      if(m_position.PositionType()==POSITION_TYPE_BUY)
        {
         if(LongModified())
            return(true);
        }
      else 
        {
         if(ShortModified())
            return(true);
        }
     }
     return(false);
  }
bool CExpert::ShortModified(void)
  {
   bool   res=false;
//--- check for trailing stop
   if(TrailingStop>0)
     {
      if((m_position.PriceOpen()-m_symbol.Ask())>(m_adjusted_point*TrailingStop))
        {
         double sl=NormalizeDouble(m_symbol.Ask()+m_traling_stop,m_symbol.Digits());
         double tp=m_position.TakeProfit();
         if(m_position.StopLoss()>sl || m_position.StopLoss()==0.0)
           {
            if(sl>m_position.PriceOpen()) return(false);
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Short position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           }
        }
     }
//--- result
   return(res);
  }
bool CExpert::LongModified(void)
  {
   bool res=false;
//--- check for trailing stop
   if(TrailingStop>0)
     {
      if(m_symbol.Bid()-m_position.PriceOpen()>m_adjusted_point*TrailingStop)
        {
         double sl=NormalizeDouble(m_symbol.Bid()-m_traling_stop,m_symbol.Digits());
         double tp=m_position.TakeProfit();
         if(m_position.StopLoss()<sl || m_position.StopLoss()==0.0)
           {
            if (sl<m_position.PriceOpen()) return(false);
            //--- modify position
            if(m_trade.PositionModify(Symbol(),sl,tp))
               printf("Long position by %s to be modified",Symbol());
            else
              {
               printf("Error modifying position by %s : '%s'",Symbol(),m_trade.ResultComment());
               printf("Modify parameters : SL=%f,TP=%f",sl,tp);
              }
            //--- modified and must exit from expert
            res=true;
           }
        }
     }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CExpert::Processing(void)
  {
   //--- refresh indicators
   if(BarsCalculated(m_handle_lelExt)<2)
      return(false);
   if(CopyBuffer(m_handle_lelExt,0,0,2,m_buff_lelExt0)  !=2 ||
      CopyBuffer(m_handle_lelExt,1,0,2,m_buff_lelExt1)  !=2)
      return(false);
   ArraySetAsSeries(m_buff_lelExt0,0);
   ArraySetAsSeries(m_buff_lelExt1,0);
   m_lelExt0 = m_buff_lelExt0[1];
   m_lelExt1 = m_buff_lelExt1[1];
   Print("new bar:",TimeCurrent()," ",DoubleToString(m_lelExt0,5),"/",DoubleToString(m_lelExt1,5));
   //--- getting resulting Value
   int resultIndi = 0;
   if(m_lelExt0 != EMPTY_VALUE && m_lelExt0 > 0){
      Print(__LINE__," ",DoubleToString(m_lelExt0,m_symbol.Digits()));
      resultIndi = 1;
   }
   if(m_lelExt1 != EMPTY_VALUE && m_lelExt1 > 0){
      Print(__LINE__," ",DoubleToString(m_lelExt1,m_symbol.Digits()));
      resultIndi =-1;
   }
   if(resultIndi == 0)
      return(false);
   //--- sending orders
   if(m_position.Select(Symbol()))
     {
      if(m_position.PositionType()==POSITION_TYPE_BUY)
        {
         if(resultIndi<0)
           {
            if(LongClosed()&&
               ShortOpened())
            return(true);
           }
        }
      else 
        {
         if(resultIndi>0)
           {
            if(ShortClosed() &&
               LongOpened())
            return(true);
           }
        }
     }
   else
     {
      if(resultIndi>0)
         if(LongOpened())
            return(true);
      if(resultIndi<0)
         if(ShortOpened())
            return(true);   
     }
   return(false);
  }
bool CExpert::LongClosed(void)
  {
   if(m_trade.PositionClose(Symbol()))
     {
      printf("Long position by %s to be closed",Symbol());
      return(true);
     }
   else
      printf("Error closing position by %s : '%s'",Symbol(),m_trade.ResultComment());
            //--- processed and cannot be modified
   return(false);
  }
bool CExpert::ShortClosed(void)
  {
   if(m_trade.PositionClose(Symbol()))
     {
      printf("Long position by %s to be closed",Symbol());
      return(true);
     }
   else
      printf("Error closing position by %s : '%s'",Symbol(),m_trade.ResultComment());
            //--- processed and cannot be modified
   return(false);
  }
bool CExpert::LongOpened(void)
  {
   double price=m_symbol.Ask();
   double tp   =m_symbol.Bid()+m_take_profit;
   //--- check for free money
   if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_BUY,Lot,price)<0.0)
      printf("We have no money. Free Margin = %f",m_account.FreeMargin());
   else
     {
     //--- open position
      if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_BUY,Lot,price,0.0,tp))
         printf("Position by %s to be opened",Symbol());
      else
        {
         printf("Error opening BUY position by %s : '%s'",Symbol(),m_trade.ResultComment());
         printf("Open parameters : price=%f,TP=%f",price,tp);
        }
     }
   return(true); 
  }
bool CExpert::ShortOpened(void)
  {
   double price=m_symbol.Bid();
   double tp   =m_symbol.Ask()-m_take_profit;
   //--- check for free money
   if(m_account.FreeMarginCheck(Symbol(),ORDER_TYPE_SELL,Lot,price)<0.0)
      printf("We have no money. Free Margin = %f",m_account.FreeMargin());
   else
     {
     //--- open position
      if(m_trade.PositionOpen(Symbol(),ORDER_TYPE_SELL,Lot,price,0.0,tp))
         printf("Position by %s to be opened",Symbol());
      else
        {
         printf("Error opening SELL position by %s : '%s'",Symbol(),m_trade.ResultComment());
         printf("Open parameters : price=%f,TP=%f",price,tp);
        }
     }
   return(true);
  }
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- create all necessary objects
   if(!ResultExpert.Init())
      return(INIT_FAILED);
//--- succeed
   return(INIT_SUCCEEDED);
  }
void OnDeinit(const int reason)  {  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   static int perSec = PeriodSeconds(PERIOD_CURRENT);
   static datetime limit_time=0; // last trade processing time + timeout
//--- don't process if timeout
   if(TimeCurrent()>=limit_time){
      //--- change limit time by timeout in seconds if processed
      if(ResultExpert.ProcessingTrail())
         limit_time=TimeCurrent()+ExtTimeOut;
   }
   static datetime last_bar=0; // last bar datetime
   if (TimeCurrent()/perSec*perSec > last_bar){
      ResultExpert.Processing();
      last_bar = TimeCurrent()/perSec*perSec;
   }
  }
//+------------------------------------------------------------------+
/* lot amount check, need into processing order send
//--- check for right lots amount
   
*/ 
double CExpert::LotSize()
{
   static int coefficient = 0;   
   static datetime lastOpen;
   
   HistorySelect(0,TimeCurrent());
   uint deals = HistoryDealsTotal();
   ulong ticket;
   for(uint i = 0; i < deals; i++){
      if ((ticket = HistoryDealGetTicket(i)) >= 0){
         datetime timeDeal = (datetime)HistoryDealGetInteger(ticket, DEAL_TIME);
         if(timeDeal > lastOpen){
            Print(__LINE__," new deal:",ticket," @",TimeToString(timeDeal));
            lastOpen = timeDeal;
         }
      }
   
   }
   
   
   
   if(Lot<m_symbol.LotsMin() || Lot>m_symbol.LotsMax())
     {
      printf("Lots amount must be in the range from %f to %f",m_symbol.LotsMin(),m_symbol.LotsMax());
      return(false);
     }
   if(MathAbs(Lot/m_symbol.LotsStep()-MathRound(Lot/m_symbol.LotsStep()))>1.0E-10)
     {
      printf("Lots amount is not corresponding with lot step %f",m_symbol.LotsStep());
      return(false);
     }
   return Lot;
}
